const toArray = (target) => {
    return [...target.values()];
};
const toMap = (target) => {
    return target;
};
const getLength = (target) => {
    return target.size;
};
const getKeys = (target) => {
    return target.keys();
};
const getValues = (target) => {
    return target.values();
};
const forEach = (target, cfn) => {
    for (let [k, v] of target.entries()) {
        cfn(v, k);
    }
};
const sortBy = (target, compareFn) => {
    const arr = Array.from(target);
    arr.sort((a, b) => compareFn(a[1], b[1]));
    return makeKMap(new Map(arr.map((i) => [i[0], i[1]])));
};
const paginate = (target, currentPpage, num, compareFn) => {
    const res = makeKMap();
    let data;
    if (undefined === compareFn)
        data = target;
    else
        data = sortBy(target, compareFn).toMap();
    if (currentPpage < 1)
        currentPpage = 1;
    const min = (currentPpage - 1) * num;
    const max = min + num;
    let i = 0;
    for (const [key, value] of data) {
        i++;
        if (i <= min)
            continue;
        if (i > max)
            break;
        res[key] = value;
    }
    return res;
};
const getAtKey = (target, index) => {
    if (index < 0)
        index = target.size - Math.abs(index);
    let i = 0;
    for (const [key, value] of target) {
        if (i === index)
            return key;
        i++;
    }
    return undefined;
};
const getAtValue = (target, index) => {
    if (index < 0)
        index = target.size - Math.abs(index);
    let i = 0;
    for (const [key, value] of target) {
        if (i === index)
            return value;
        i++;
    }
    return undefined;
};
export function makeKMap(map) {
    if (undefined === map)
        map = new Map();
    return new Proxy(map, {
        get: (target, key, receiver) => {
            if (!key)
                return undefined;
            else if ('toArray' === key) {
                return () => toArray(target);
            }
            else if ('toMap' === key) {
                return () => toMap(target);
            }
            else if ('getLength' === key) {
                return () => getLength(target);
            }
            else if ('getKeys' === key) {
                return () => getKeys(target);
            }
            else if ('getValues' === key) {
                return () => getValues(target);
            }
            else if ('forEach' === key) {
                return (cfn) => forEach(target, cfn);
            }
            else if ('sortBy' === key) {
                return (compareFn) => sortBy(target, compareFn);
            }
            else if ('paginate' === key) {
                return (currentPpage, num, compareFn) => paginate(target, currentPpage, num, compareFn);
            }
            else if ('getAtKey' === key) {
                return (index) => getAtKey(target, index);
            }
            else if ('getAtValue' === key) {
                return (index) => getAtValue(target, index);
            }
            else
                return target.get(key);
        },
        set: (target, key, value) => (target.set(key, value), true),
        has: (target, key) => target.has(key),
        deleteProperty: (target, key) => target.delete(key),
        ownKeys: (target) => [...target.keys()],
        getOwnPropertyDescriptor: (target, key) => ({
            enumerable: true,
            configurable: true,
            value: target.get(key),
        }),
    });
}
//# sourceMappingURL=make-kurimudb-map.func.js.map